/*
 * Copyright (C) 1994 Jettero Heller
 * Copyright (C) 1994, 1995 Peter Williams
 *
 * You may distribute under the terms of the GNU Library General Public 
 * License as specified in the COPYING file that comes with the Xarm library.
 *
 */
/* Property.h,v 1.5 1995/10/19 12:44:30 pgw Exp */

#if !defined(_XARM_PROPERTY_H_)
#define _XARM_PROPERTY_H_

/*
 * Warning:  These classes diverge from the Motif 1.2 standard GUI and 
 * use widgets written by the author.
 */

/* ======================================================================= */
/* Include(s): */
#include <Xmext/Property.h>
#include <Xarm/BulletinB.h>

class PropertyPageClass : public BulletinBoardClass {
 protected:
   PropertyPageClass() {} /* only a derived class can make this */
   virtual WidgetClass classPointer() { return xmPropertyPageWidgetClass; }
   virtual _XtString className() { return (_XtString) "XmPropertyPage"; }
 public:

   /* Callbacks */
   /* activate callbacks for the following default buttons */
   void cancelCallback(XtCallbackProc fp, XtPointer udata = NULL);
   void okCallback(XtCallbackProc fp, XtPointer udata = NULL);
   void applyCallback(XtCallbackProc fp, XtPointer udata = NULL);

   /* Called when the current page is no longer active. */
   void killActiveCallback(XtCallbackProc fp, XtPointer udata = NULL);
   /* Called when the page is made the active page. */
   void setActiveCallback(XtCallbackProc fp, XtPointer udata = NULL);

   /* extras */
   void setModified(Boolean b = TRUE); /* enable or disable apply button */
   void cancelLabelString(char *s);  /* changes text of cancel button */
   
};

class PropertyPage : public PropertyPageClass {
 public:
   PropertyPage(Widget parent, 
                 ArgList arglist = NULL, 
                 Cardinal cnt = 0, 
                 _XtString name = NULL);
   PropertyPage(_XtString name, Widget parent); 
};

class PropertySheetClass : public BulletinBoardClass {
 protected:
   PropertySheetClass() {} 
   virtual WidgetClass classPointer() { return xmPropertySheetWidgetClass; }
   virtual _XtString className() { return (_XtString) "XmPropertySheet"; }
 public:
   /* callbacks */
   /* activate callbacks for the following default buttons */
   void cancelCallback(XtCallbackProc fp, XtPointer udata = NULL);
   void okCallback(XtCallbackProc fp, XtPointer udata = NULL);
   void applyCallback(XtCallbackProc fp, XtPointer udata = NULL);

   /* resources */
   int pageCount(); /* return number of pages in the sheet */
   void cancelLabelString(char *s);  /* changes text of cancel button */
   char *cancelLabelString();
   void okLabelString(char *s);
   char *okLabelString();
   void applyLabelString(char *s);
   char *applyLabelString();

   /* convience routines */
   Widget getPage(int p);  /* get page returns a PropertyPage widget */
   void addPage(Widget page, int loc = 0); /* add a PropertyPage widget */
   void removePage(int p); /* remove a page */
   void removePage(Widget page);  /* dito */

   Widget getChild(unsigned char);

};

class PropertySheet : public PropertySheetClass {
 public:
   PropertySheet(Widget parent, 
                 ArgList arglist = NULL, 
                 Cardinal cnt = 0, 
                 _XtString name = NULL);
   PropertySheet(_XtString name, Widget parent); 
};

class PropertySheetDialog : public PropertySheetClass {
 public:
   PropertySheetDialog(Widget parent, 
                 ArgList arglist = NULL, 
                 Cardinal cnt = 0, 
                 _XtString name = NULL);
   PropertySheetDialog(_XtString name, Widget parent); 
};

#endif
