/*
 * Copyright (C) 1994 Jettero Heller
 * Copyright (C) 1994, 1995 Peter Williams
 *
 * You may distribute under the terms of the GNU Library General Public 
 * License as specified in the COPYING file that comes with the Xarm library.
 *
 */
/* MessageB.C,v 1.4 1995/10/19 12:45:07 pgw Exp */
#include <Xarm/MessageB.h>

/* MessageBoxClass Members */
/* convenience routines */
Widget
MessageBoxClass::getChild(unsigned char child)
{
   return XmMessageBoxGetChild(widget(), child);
}

/* callbacks */
void
MessageBoxClass::cancelCallback(XtCallbackProc funcp, XtPointer data)
{
   addCallback(XmNcancelCallback,funcp,data);
}

void
MessageBoxClass::okCallback(XtCallbackProc funcp, XtPointer data)
{
   addCallback(XmNokCallback,funcp,data);
}

/* resources */
void
MessageBoxClass::cancelLabelString(char *s)
{
   XmString xmstr = cvtStringToXmString(s);
   XtVaSetValues(widget(), XmNcancelLabelString, xmstr, NULL);
   XmStringFree(xmstr);
}

char *
MessageBoxClass::cancelLabelString()
{
   XmString xmstr;
   XtVaGetValues(widget(), XmNcancelLabelString, &xmstr, NULL);
   return cvtXmStringToString(xmstr);
}

void
MessageBoxClass::defaultButtonType(unsigned char c)
{
   XtVaSetValues(widget(), XmNdefaultButtonType, c, NULL);
}

unsigned char
MessageBoxClass::defaultButtonType()
{
   unsigned char c;
   XtVaGetValues(widget(), XmNdefaultButtonType, &c, NULL);
   return c;
}

void
MessageBoxClass::dialogType(unsigned char c)
{
   XtVaSetValues(widget(), XmNdialogType, c, NULL);
}

unsigned char
MessageBoxClass::dialogType()
{
   unsigned char c;
   XtVaGetValues(widget(), XmNdialogType, &c, NULL);
   return c;
}

void
MessageBoxClass::messageAlignment(unsigned char c)
{
   XtVaSetValues(widget(), XmNmessageAlignment, c, NULL);
}

unsigned char
MessageBoxClass::messageAlignment()
{
   unsigned char c;
   XtVaGetValues(widget(), XmNmessageAlignment, &c, NULL);
   return c;
}

void
MessageBoxClass::messageString(char *s)
{
   XmString xmstr = cvtStringToXmString(s);
   XtVaSetValues(widget(), XmNmessageString, xmstr, NULL);
   XmStringFree(xmstr);
}

char *
MessageBoxClass::messageString()
{
   XmString xmstr;
   XtVaGetValues(widget(), XmNmessageString, &xmstr, NULL);
   return cvtXmStringToString(xmstr);
}

void
MessageBoxClass::minimizeButtons(Boolean b)
{
   XtVaSetValues(widget(), XmNminimizeButtons, b, NULL);
}

Boolean
MessageBoxClass::minimizeButtons()
{
   Boolean b;
   XtVaGetValues(widget(), XmNminimizeButtons, &b, NULL);
   return b;
}

char *
MessageBoxClass::okLabelString()
{
   XmString xmstr;
   XtVaGetValues(widget(), XmNokLabelString, &xmstr, NULL);
   return cvtXmStringToString(xmstr); 
}

void
MessageBoxClass::okLabelString(char *s)
{
   XmString xmstr = cvtStringToXmString(s);
   XtVaSetValues(widget(), XmNokLabelString, xmstr, NULL);
   XmStringFree(xmstr); 
}

void
MessageBoxClass::symbolPixmap(Pixmap pixmap)
{
   XtVaSetValues(widget(), XmNsymbolPixmap, pixmap, NULL);
}

Pixmap
MessageBoxClass::symbolPixmap()
{
   Pixmap pixmap;
   XtVaGetValues(widget(), XmNsymbolPixmap, &pixmap, NULL);
   return pixmap;
}

/* constructor */
Widget
MessageBox::createWidget(_XtString name, 
                           Widget parent, 
                           ArgList arglist,
                           Cardinal cnt)
{
   if (name == NULL || *name == '\0') { name = className(); }
   Widget w = XmCreateMessageBox(parent, name, arglist, cnt);
}

MessageBox::MessageBox(Widget parent, char *mesg, _XtString name)
{
   Widget w = createWidget(name, parent);
   widget(w);
   messageString(mesg);
   manage();
}

MessageBox::MessageBox(Widget parent,
                       ArgList arglist,
                       Cardinal cnt,
                       _XtString name)
{
   Widget w = createWidget(name, parent, arglist, cnt);

   widget(w);
   manage();
}

MessageBox::MessageBox(_XtString name, Widget parent)
{
   Widget w = createWidget(name, parent);
   widget(w);
   manage();
}

Widget
MessageDialog::createWidget(_XtString name, 
                           Widget parent, 
                           ArgList arglist,
                           Cardinal cnt)
{
   if (name == NULL || *name == '\0') { name = className(); }
   Widget w = XmCreateMessageDialog(parent, name, arglist, cnt);
}

MessageDialog::MessageDialog(Widget parent, char *mesg, _XtString name)
{
   Widget w = createWidget(name,parent);
   widget(w);
   messageString(mesg);
}

MessageDialog::MessageDialog(Widget parent,
                             ArgList arglist,
                             Cardinal cnt,
                             _XtString name)
{
   Widget w = createWidget(name,parent, arglist, cnt);
   widget(w);
}

MessageDialog::MessageDialog(_XtString name, Widget parent)
{
   Widget w = createWidget(name,parent);
   widget(w);
}

Widget
InformationDialog::createWidget(_XtString name, 
                                Widget parent, 
                                ArgList arglist,
                                Cardinal cnt)
{
   if (name == NULL || *name == '\0') { name = className(); }
   return XmCreateInformationDialog(parent, name, arglist, cnt);
}

InformationDialog::InformationDialog(Widget parent, char *mesg, _XtString name)
{
   Widget w = createWidget(name, parent);
   widget(w);
   messageString(mesg);
}

InformationDialog::InformationDialog(Widget parent,
                                     ArgList arglist,
                                     Cardinal cnt,
                                     _XtString name)
{
   Widget w = createWidget(name, parent, arglist, cnt);

   widget(w);
}

InformationDialog::InformationDialog(_XtString name, Widget parent)
{
   Widget w = createWidget(name, parent);

   widget(w);
}

Widget
QuestionDialog::createWidget(_XtString name, 
                          Widget parent, 
                          ArgList arglist,
                          Cardinal cnt)
{
   if (name == NULL || *name == '\0') { name = className(); }
   return XmCreateQuestionDialog(parent, name, arglist, cnt);
}

QuestionDialog::QuestionDialog(Widget parent, char *mesg, _XtString name)
{
   Widget w = createWidget(name, parent);
   widget(w);
   messageString(mesg);
}

QuestionDialog::QuestionDialog(Widget parent,
                               ArgList arglist,
                               Cardinal cnt,
                               _XtString name)
{
   Widget w = createWidget(name, parent, arglist, cnt);
   widget(w);
}

Widget
WarningDialog::createWidget(_XtString name, 
                          Widget parent, 
                          ArgList arglist,
                          Cardinal cnt)
{
   if (name == NULL || *name == '\0') { name = className(); }
   return XmCreateWarningDialog(parent, name, arglist, cnt);
}

WarningDialog::WarningDialog(Widget parent, char *mesg, _XtString name)
{
   Widget w = createWidget(name, parent);
   widget(w);
   messageString(mesg);
}

WarningDialog::WarningDialog(Widget parent, 
                             ArgList arglist,
                             Cardinal cnt,
                             _XtString name)
{
   Widget w = createWidget(name, parent, arglist, cnt);
   widget(w);
}

WarningDialog::WarningDialog(_XtString name, Widget parent)
{
   Widget w = createWidget(name, parent);
   widget(w);
}

Widget
WorkingDialog::createWidget(_XtString name, 
                          Widget parent, 
                          ArgList arglist,
                          Cardinal cnt)
{
   if (name == NULL || *name == '\0') { name = className(); }
   return XmCreateWorkingDialog(parent, name, arglist, cnt);
}

WorkingDialog::WorkingDialog(Widget parent, char *mesg, _XtString name)
{
   Widget w = createWidget(name, parent);
   widget(w);
   messageString(mesg);
}

WorkingDialog::WorkingDialog(Widget parent,
                             ArgList arglist,
                             Cardinal cnt,
                             _XtString name)
{
   Widget w = createWidget(name, parent, arglist, cnt);
   widget(w);
}

WorkingDialog::WorkingDialog(_XtString name, Widget parent)
{
   Widget w = createWidget(name, parent);
   widget(w);
}

Widget
ErrorDialog::createWidget(_XtString name, 
                          Widget parent, 
                          ArgList arglist,
                          Cardinal cnt)
{
   if (name == NULL || *name == '\0') { name = className(); }
   return XmCreateErrorDialog(parent, 
                              name,
                              arglist,
                              cnt);
}

ErrorDialog::ErrorDialog(Widget parent, char *mesg, _XtString name)
{
   Widget w = createWidget(name, parent);
   widget(w);
   messageString(mesg);
}

ErrorDialog::ErrorDialog(Widget parent, 
                         ArgList arglist,
                         Cardinal cnt,
                         _XtString name)
{
   Widget w = createWidget(name, parent, arglist, cnt); 
   widget(w);
}

ErrorDialog::ErrorDialog(_XtString name, Widget parent)
{
   Widget w = createWidget(name, parent); 
   widget(w);
}

