/*
 * Copyright (C) 1994 Jettero Heller
 * Copyright (C) 1994, 1995 Peter Williams
 *
 * You may distribute under the terms of the GNU Library General Public 
 * License as specified in the COPYING file that comes with the Xarm library.
 *
 */
/* Property.C,v 1.6 1995/10/19 12:45:09 pgw Exp */
#include <Xarm/Property.h>

/* PropertyPageClass members */
/* callbacks */
void
PropertyPageClass::cancelCallback(XtCallbackProc funcp, XtPointer data)
{
   addCallback(XmNcancelCallback,funcp,data);
}

void
PropertyPageClass::okCallback(XtCallbackProc funcp, XtPointer data)
{
   addCallback(XmNokCallback,funcp,data);
}

void
PropertyPageClass::applyCallback(XtCallbackProc funcp, XtPointer data)
{
   addCallback(XmNapplyCallback,funcp,data);
}

/* resources */
void
PropertyPageClass::cancelLabelString(char *s)
{
   Widget w = XmPropertySheetGetChild(XtParent(widget()),
                                      XmDIALOG_CANCEL_BUTTON); 
   if(w != NULL)
   {
      XmString xmstr = cvtStringToXmString(s);
      XtVaSetValues(w, XmNcancelLabelString, xmstr, NULL);
      XmStringFree(xmstr);
   }
   else
   {
      XtAppWarning(appContext(), "Could not find the cancel button on the "
                   "PropertyPage parent widget (which should be a Property "
                   "Sheet).");
   }
}

void
PropertyPageClass::setModified(Boolean b)
{
   Widget w = XmPropertySheetGetChild(XtParent(widget()), XmDIALOG_APPLY_BUTTON); 

   if(w != NULL)
   {
      XtSetSensitive(w, b);
   }
   else
   {
      XtAppWarning(appContext(), "Could not find the apply button on the "
                   "PropertyPage parent widget (which should be a Property "
                   "Sheet).");
   }
}

PropertyPage::PropertyPage(Widget parent, ArgList arglist, Cardinal cnt,
                             _XtString name)
{
   if(name == NULL || *name == '\0') { name = className(); }
   Widget w = XmCreatePropertyPage(parent, name, arglist, cnt);
   widget(w);
}

PropertyPage::PropertyPage(_XtString name, Widget parent)
{
   if(name == NULL || *name == '\0') { name = className(); }
   Widget w = XmCreatePropertyPage(parent, name, NULL, 0);
   widget(w);
}

/* PropertySheetClass members */
Widget
PropertySheetClass::getChild(unsigned char ch)
{
   return XmPropertySheetGetChild(widget(), ch);
}

int
PropertySheetClass::pageCount()
{
   int cnt;
   XtVaGetValues(widget(), XmNpageCount, &cnt, NULL);
   return cnt;
}

Widget 
PropertySheetClass::getPage(int pos)
{
   return XmPropertySheetGetPage(widget(), pos);
}

void
PropertySheetClass::addPage(Widget w, int pos)
{
   XmPropertySheetAddPage(widget(), w, pos);
}

void
PropertySheetClass::removePage(Widget w)
{
   XmPropertySheetRemovePage(widget(), w);
}

void
PropertySheetClass::removePage(int pos)
{
   XmPropertySheetRemovePos(widget(), pos);
}

void
PropertySheetClass::cancelLabelString(char *s)
{
   XmString xmstr = cvtStringToXmString(s);
   XtVaSetValues(widget(), XmNcancelLabelString, xmstr, NULL);
   XmStringFree(xmstr);
}

char *
PropertySheetClass::cancelLabelString()
{
   XmString xmstr;
   XtVaGetValues(widget(), XmNcancelLabelString, &xmstr, NULL);
   return cvtXmStringToString(xmstr);
}

void
PropertySheetClass::applyLabelString(char *s)
{
   XmString xmstr = cvtStringToXmString(s);
   XtVaSetValues(widget(), XmNapplyLabelString, xmstr, NULL);
   XmStringFree(xmstr);
}

char *
PropertySheetClass::applyLabelString()
{
   XmString xmstr;
   XtVaGetValues(widget(), XmNapplyLabelString, &xmstr, NULL);
   return cvtXmStringToString(xmstr);
}

void
PropertySheetClass::okLabelString(char *s)
{
   XmString xmstr = cvtStringToXmString(s);
   XtVaSetValues(widget(), XmNokLabelString, xmstr, NULL);
   XmStringFree(xmstr);
}

char *
PropertySheetClass::okLabelString()
{
   XmString xmstr;
   XtVaGetValues(widget(), XmNokLabelString, &xmstr, NULL);
   return cvtXmStringToString(xmstr);
}

void
PropertySheetClass::cancelCallback(XtCallbackProc funcp, XtPointer data)
{
   addCallback(XmNcancelCallback,funcp,data);
}

void
PropertySheetClass::okCallback(XtCallbackProc funcp, XtPointer data)
{
   addCallback(XmNokCallback,funcp,data);
}

void
PropertySheetClass::applyCallback(XtCallbackProc funcp, XtPointer data)
{
   addCallback(XmNapplyCallback,funcp,data);
}

PropertySheet::PropertySheet(Widget parent, ArgList arglist, Cardinal cnt,
                             _XtString name)
{
   if(name == NULL || *name == '\0') { name = className(); }
   Widget w = XmCreatePropertySheet(parent, name, arglist, cnt);
   widget(w);
}

PropertySheet::PropertySheet(_XtString name, Widget parent)
{
   if(name == NULL || *name == '\0') { name = className(); }
   Widget w = XmCreatePropertySheet(parent, name, NULL, 0);
   widget(w);
}

PropertySheetDialog::PropertySheetDialog(Widget parent, ArgList arglist, Cardinal cnt,
                             _XtString name)
{
   if(name == NULL || *name == '\0') { name = className(); }
   Widget w = XmCreatePropertySheet(parent, name, arglist, cnt);
   widget(w);
}

PropertySheetDialog::PropertySheetDialog(_XtString name, Widget parent)
{
   if(name == NULL || *name == '\0') { name = className(); }
   Widget w = XmCreatePropertySheet(parent, name, NULL, 0);
   widget(w);
}

