/* Property.c,v 1.6 1995/10/02 15:06:27 pgw Exp */

#include <Xmext/PropertyP.h>
#include <Xm/DialogS.h>
#include <Xm/PushB.h>
#include <stdio.h>

/* PropertySheet widget */
#define TAB_RESOURCE_NAME "XmPropertySheetTab"
#define Unused(x) (void)(x)
#define Offset(field) XtOffsetOf(XmPropertySheetRec, property_sheet.field)

/* local declarations */
static void activate_cb(Widget w, XtPointer udata, XtPointer cdata); 
static void change_managed(Widget w);
static void class_initialize();
static void class_part_initialize(WidgetClass class);
static void destroy(Widget w);
#if 0
static void expose(Widget w, XEvent *event, Region region);
#endif
static XtGeometryResult geometry_manager(Widget w, 
                                         XtWidgetGeometry *request, 
                                         XtWidgetGeometry *reply);
static void initialize(Widget request, 
                       Widget c_new,
                       ArgList args,
                       Cardinal *num_args);
static void kill_active_page(Widget sheet);
static void layout(XmPropertySheetWidget w);
static int page_position(Widget sheet, Widget page);
static XtGeometryResult query_geometry(Widget w, 
                                      XtWidgetGeometry *proposed, 
                                      XtWidgetGeometry *answer);
/* realize sets the active page & tab if not null */
static void realize(Widget w,
                    XtValueMask *value_mask,
                    XSetWindowAttributes *attributes);
static void set_active_page(Widget sheet, Widget page);
static void set_active_pos(Widget sheet, int pos);
static Boolean set_values(Widget current, 
                          Widget request, 
                          Widget c_new, 
                          ArgList args,
                          Cardinal *num_args);
static void tab_activate_cb(Widget w, XtPointer udata, XtPointer cdata);

/* locale functions */
static void _XmScancelLabelString(Widget wid, int offset, XrmValue *value);
static void _XmSapplyLabelString(Widget wid, int offset, XrmValue *value);
static void _XmSokLabelString(Widget wid, int offset, XrmValue *value);

/* ========================================================================== */
/* Resources for the Property Sheet class */
static XtResource resources[] = {

   {
     XmNactivePos,
     XmCActivePos, 
     XmRInt,
     sizeof(int),
     Offset(active_pos),
     XmRImmediate,
     0
   },

   { 
     XmNapplyLabelString, /* resource_name: specify using XtN symbol */
     XmCApplyLabelString, /* resource_class: specify using XtC symbol */
     XmRXmString,          /* resource_type: actual data type of variable */
     sizeof(XmString),     /* resource_size: specify using sizeof() */
     Offset(apply_label_string), 
                           /* resource_offset: specify using XtOffsetOf() */ 
     XtRCallProc,          /* default_type: will be converted to resrc_type */
     (XtPointer) _XmSapplyLabelString 
                           /* default_address: address of default value */
    },

   {
     XmNapplyCallback,
     XmCCallback, 
     XtRCallback,
     sizeof(XtCallbackList),
     Offset(apply_callback),
     XmRImmediate,
     NULL
   },

   {
     XmNcancelLabelString, 
     XmCCancelLabelString, 
     XmRXmString, 
     sizeof(XmString),
     Offset(cancel_label_string), 
     XtRCallProc, 
     (XtPointer) _XmScancelLabelString
    },

   {
     XmNcancelCallback,
     XmCCallback, 
     XtRCallback,
     sizeof(XtCallbackList),
     Offset(cancel_callback),
     XmRImmediate,
     NULL
   },

   {
     XmNcornerStyle,
     XmCCornerStyle,
     XtRUnsignedChar,
     sizeof(unsigned char),
     Offset(corner_style),
     XtRImmediate,
     (XtPointer) XmCORNER_ARC
   },

   {
     XmNokLabelString, 
     XmCOkLabelString, 
     XmRXmString, 
     sizeof(XmString),
     Offset(ok_label_string), 
     XtRCallProc, 
     (XtPointer) _XmSokLabelString
   }, 

   {
     XmNokCallback,
     XmCCallback, 
     XtRCallback,
     sizeof(XtCallbackList),
     Offset(ok_callback),
     XmRImmediate,
     NULL
   },

   {
     XmNpageCount,
     XmCPageCount,
     XmRInt,
     sizeof(int),
     Offset(page_count),
     XmRImmediate,
     (XtPointer) 0
   },

   {
     XmNpages,
     XmCPages, 
     XmRWidgetList,
     sizeof(WidgetList),
     Offset(pages),
     XmRImmediate,
     NULL
   },

   {
     XmNtabs,
     XmCTabs, 
     XmRWidgetList,
     sizeof(WidgetList),
     Offset(tabs),
     XmRImmediate,
     NULL
   }

};

/* ========================================================================== */
/* Class record */

XmPropertySheetClassRec xmPropertySheetClassRec = {
   /* Core class part */
   {
      (WidgetClass) &xmBulletinBoardClassRec, /* superclass */
      "XmPropertySheet",                      /* class_name */
      sizeof(XmPropertySheetRec),             /* widget_size */
      class_initialize,                       /* class_initialize */
      class_part_initialize,                  /* class_part_initialize */
      FALSE,                                  /* class_inited */
      initialize,                             /* initialize */
      NULL,                                   /* initialize_hook */
      realize,                                /* realize */
      NULL,                                   /* actions */
      0,                                      /* num_actions */
      resources,                              /* resources */
      XtNumber(resources),                    /* num_resources */
      NULLQUARK,                              /* xrm_class */
      TRUE,                                   /* compress_motion */
      XtExposeCompressMultiple,               /* compress_exposure */
      TRUE,                                   /* compress_enterleave */
      FALSE,                                  /* visible_interest */
      destroy,                                /* destroy */
      NULL,                                   /* resize */
      XtInheritExpose,                        /* expose */
      set_values,                             /* set_values */
      NULL,                                   /* set_values_hook */
      XtInheritSetValuesAlmost,               /* set_values_almost */
      NULL,                                   /* get_values_hook */
      NULL,                                   /* accept_focus */
      XtVersion,                              /* version */
      NULL,                                   /* callback offsets */
      XtInheritTranslations,                  /* tm_table */
      query_geometry,                         /* query_geometry */
      NULL,                                   /* display_accelerator */
      NULL                                    /* extension */ 
   },
   /* Composite class part */
   {
      geometry_manager,       /* geometry manager */
      change_managed,         /* change_managed */
      XtInheritInsertChild,   /* insert_child */ 
      XtInheritInsertChild,   /* delete_child */ 
      NULL                    /* extension */ 
   },
   /* Constraint class part */
   {
      NULL,      /* FIX ME */ /* subresources */
      0,         /* FIX ME */ /* subresource_count */
      0,         /* FIX ME */ /* constraint_size */
      NULL,      /* FIX ME */ /* initialize */
      NULL,      /* FIX ME */ /* destroy */
      NULL,      /* FIX ME */ /* set_values */
      NULL                    /* extension */ 
   },
   /* XmManager class part */
   {
      NULL,                   /* translations */ 
      NULL,                   /* syn_resources */ 
      0,                      /* num_syn_resources */
      NULL,                   /* syn_constraint_resources */ 
      0,                      /* num_syn_constraint_resources */
      XmInheritParentProcess, /* parent process */
      NULL                    /* extension */ 
   },
   /* XmBulletin class part */
   {
      (Dimension) NULL                    /* extension */ 
   },
   /* XmPropertySheet class part */
   {
      NULL                    /* extension */ 
   }
};

WidgetClass xmPropertySheetWidgetClass = (WidgetClass) &xmPropertySheetClassRec;

/* ========================================================================== */
/* locale functions */

static void
_XmScancelLabelString(Widget wid, int offset, XrmValue *value)
{
   /* here is the default for the C locale */
   value->addr = (XtPointer) "Cancel";
   /* 
    * Read value from array table or file if not in C locale 
    * FIX ME 
    */
}

static void
_XmSapplyLabelString(Widget wid, int offset, XrmValue *value)
{
   /* here is the default for the C locale */
   value->addr = (XtPointer) "Apply";
   /* 
    * Read value from array table or file if not in C locale 
    * FIX ME 
    */
}

static void
_XmSokLabelString(Widget wid, int offset, XrmValue *value)
{
   /* here is the default for the C locale */
   value->addr = (XtPointer) "OK";
   /* 
    * Read value from array table or file if not in C locale 
    * FIX ME 
    */
}

/* ========================================================================== */
/* functions */

/*
 * activate_cb
 *
 * This function is used to dispatch activate callbacks for 
 * the various button activate callbacks resources.
 */
static void
activate_cb(Widget w, XtPointer udata, XtPointer cdata)
{
   XmPropertySheetWidget ps = (XmPropertySheetWidget) XtParent(w);

   if(w == ps->property_sheet.ok_button)
   {
      XtCallCallbackList((Widget) ps,
                         ps->property_sheet.ok_callback, 
                         udata);
   }
   else if(w == ps->property_sheet.apply_button)
   {
      XtCallCallbackList((Widget) ps, 
                         ps->property_sheet.apply_callback,
                         udata);
   }
   else if(w == ps->bulletin_board.cancel_button)
   {
      XtCallCallbackList((Widget) ps,
                         ps->property_sheet.cancel_callback,
                         udata);
   }
}

static void 
change_managed(Widget w)
{
   layout((XmPropertySheetWidget)w);
}

static void
class_initialize()
{
}

static void
class_part_initialize(WidgetClass widget_class)
{
   Unused(widget_class);
}

static void
destroy(Widget w)
{
   Unused(w);
}

#if 0
static void 
expose(Widget w, XEvent *event, Region region)
{
   /* FIX ME */
   Unused(w);
   Unused(event);
   Unused(region);
}
#endif

static XtGeometryResult 
geometry_manager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
{
   /* FIX ME */
   Unused(w);
   Unused(request);
   Unused(reply);
   return (XtGeometryResult) NULL;
}

static void
initialize(Widget request,
           Widget c_new,
           ArgList args,
           Cardinal *num_args)
{
   XmPropertySheetWidget ps = (XmPropertySheetWidget)c_new;

   /* FIX ME this should depend on the locale */
   ps->property_sheet.ok_button = XmCreatePushButton(c_new, "OK", NULL, 0);
   ps->bulletin_board.default_button = ps->property_sheet.ok_button;
   ps->bulletin_board.cancel_button = XmCreatePushButton(c_new, "Cancel", NULL, 0);
   ps->property_sheet.apply_button = XmCreatePushButton(c_new, "Apply", NULL, 0);

   XtAddCallback(ps->property_sheet.ok_button, XmNactivateCallback, activate_cb, NULL);
   XtAddCallback(ps->bulletin_board.cancel_button, XmNactivateCallback, activate_cb, NULL);
   XtAddCallback(ps->property_sheet.apply_button, XmNactivateCallback, activate_cb, NULL);

   XtManageChild(ps->property_sheet.ok_button);
   XtManageChild(ps->bulletin_board.cancel_button);
   XtManageChild(ps->property_sheet.apply_button);

   /* FIX ME: does this \/ belong here? */
   ps->property_sheet.pages = NULL;
   ps->property_sheet.tabs = NULL;
   ps->property_sheet.page_count = 0;
}

/*
 * kill_active_page
 *
 * This function unmanages the active page
 * and sets the active page resource in the property sheet to null.
 */
static void
kill_active_page(Widget sheet)
{
   XmPropertySheetWidget ps;

   ps = (XmPropertySheetWidget) sheet;
   if(ps->property_sheet.active_pos > 0)
   {
      XtUnmanageChild(ps->property_sheet.pages[ps->property_sheet.active_pos-1]);
      /* zero means there is no active page */
      ps->property_sheet.active_pos = 0; 
   }
}

static void
layout(XmPropertySheetWidget w)
{
   /* FIX ME */
   Unused(w);
}

/*
 * page_position 
 *
 * This function locates the position of the page
 * widget in the widget list.  A 0 is returned on failure
 *
 */
static int 
page_position(Widget sheet, Widget page)
{
   XmPropertySheetWidget ps = (XmPropertySheetWidget)sheet;
   WidgetList pages;
   int page_cnt, i;

   pages = ps->property_sheet.pages;
   page_cnt = ps->property_sheet.page_count;

   for (i=0; i<page_cnt; i++)
   {
      if(page == pages[i])
      {
         return i+1;   
      }
   }
   return 0;
}

static XtGeometryResult 
query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
{
   /* FIX ME */
   Unused(w);
   Unused(proposed);
   Unused(answer);
   return (XtGeometryResult) NULL;
}

/* realize sets the active page & tab if not null */
static void realize(Widget w,
                    XtValueMask *value_mask,
                    XSetWindowAttributes *attributes)
{
   /* FIX ME */
   Unused(w);
   Unused(value_mask);
   Unused(attributes);
}

/*
 * remove_page
 *
 * This function removes the page from the property sheet window.
 * 
 */
static void
remove_page(Widget page)
{
   XtUnmanageChild(page);
}

/*
 * remove_tab
 *
 * This function removes the tab from the property sheet window
 * and destroys the tab widget.
 */
static void
remove_tab(Widget tab)
{
   /* unmanage the tab */
   XtUnmanageChild(tab);
   /* destroy the tab widget */
   XtDestroyWidget(tab);
}
   
/*
 * set_active_page
 *
 * This function manages the new page widget and displays it in the 
 * center of the sheet.  Also, the active page resource in the property
 * sheet is set to the new page widget.
 */
static void
set_active_page(Widget sheet, Widget page)
{
   int pos;

   pos = page_position(sheet, page);
   set_active_pos(sheet, pos);
}

static void
set_active_pos(Widget sheet, int pos)
{
   XmPropertySheetWidget ps;
   ps = (XmPropertySheetWidget) sheet;

   if(pos <= 0 || pos > ps->property_sheet.page_count)
   {
      XtAppError(XtWidgetToApplicationContext(sheet),
                 "Attempted to display a widget that is not"
                 "in my list.");
      return;
   }

   XtManageChild(ps->property_sheet.pages[pos - 1]);
   ps->property_sheet.active_pos = pos;
}

static Boolean 
set_values(Widget current, 
           Widget request, 
           Widget c_new, 
           ArgList args,
           Cardinal *num_args)
{
   /* FIX ME */
   Unused(current);
   Unused(request);
   Unused(c_new);
   Unused(args);
   Unused(num_args);
   return (Boolean) NULL;
}

/*
 * tab_activate_cb
 *
 * This function is used to dispatch the XmNsetActive callbacks and
 * the XmNkillActive callbacks to the page widget.
 */
static void 
tab_activate_cb(Widget w, XtPointer udata, XtPointer cdata)
{
   XmPropertySheetWidget sheet;
   XmPropertyPageWidget page, active_page;

   sheet = (XmPropertySheetWidget) XtParent(w);
   active_page = (XmPropertyPageWidget) 
                 sheet->property_sheet.pages[sheet->property_sheet.active_pos-1];

   /* the udata should be the page widget */
   page = (XmPropertyPageWidget) udata;

   /* if a different page was selected then */
   if(active_page != page)
   {
      XtCallCallbackList((Widget) active_page, 
                         active_page->property_page.kill_active_callback,
                         cdata);
      kill_active_page((Widget) sheet);

      XtCallCallbackList((Widget) page, 
                         page->property_page.set_active_callback,
                         cdata);

      set_active_page((Widget) sheet, (Widget) page);
   }
}

/*
 * XmCreatePropertySheet
 *
 * Creates an instance of a particular widget class or compound object.
 */
Widget 
XmCreatePropertySheet(Widget parent, String name, ArgList arglist,
                      Cardinal cnt)
{
   return XtCreateWidget(name, 
                         xmPropertySheetWidgetClass,
                         parent, 
                         arglist, 
                         cnt);
}

/*
 * XmCreatePropertySheetDialog
 *
 * Creates an instance of a particular widget class or compound object.
 * The child widget of the dialog shell is returned.
 */
Widget 
XmCreatePropertySheetDialog(Widget parent, 
                            String name, 
                            ArgList arglist,
                            Cardinal cnt)
{
   Widget shell;
   Arg args[10];
   int nargs;
   char *shell_name;

   nargs = 0;
   XtSetArg(args[nargs], XmNallowShellResize, True); nargs++;
 
   shell_name = XtMalloc(strlen(name) + strlen(XmDIALOG_SUFFIX) + 1);
   strcpy(shell_name, name);
   strcat(shell_name, XmDIALOG_SUFFIX);
                           
   shell = XmCreateDialogShell(parent, shell_name, args, nargs);

   XtManageChild(shell);
   XtFree(shell_name);

   return XmCreatePropertySheet(shell, name, arglist,  cnt);
}

/*
 * XmPropertySheetGetChild
 *
 * This functions will return the specified child of the PropertySheet widget.
 */
Widget
XmPropertySheetGetChild(Widget parent, unsigned char child)
{
   XmPropertySheetWidget ps = (XmPropertySheetWidget)parent;
   switch(child)
   {
      case XmDIALOG_CANCEL_BUTTON:
         return ps->bulletin_board.cancel_button;
         break;
      case XmDIALOG_OK_BUTTON:
         return ps->property_sheet.ok_button;
         break;
      case XmDIALOG_APPLY_BUTTON:
         return ps->property_sheet.apply_button;
         break;
      case XmDIALOG_DEFAULT_BUTTON:
         /*
          * There does not seem to be a difference between the values of
          * default_button and dynamic_default_button
          * so we will use the value of default_button
          */
         return ps->bulletin_board.default_button;
         break;
      default:
         /* bad value for child */
         return NULL;
   }
}

/*
 * XmPropertySheetGetPage
 * 
 * This function returns the page widget at the specified location.
 * If the specified position is not in the sheet then a NULL is returned.
 */
Widget
XmPropertySheetGetPage(Widget sheet, int position)
{
   
   XmPropertySheetWidget ps = (XmPropertySheetWidget)sheet;

   if(position > ps->property_sheet.page_count || position <= 0)
   {
      return (Widget) NULL;
   }

   return (Widget) (ps->property_sheet.pages[position - 1]);
}

/*
 * XmPropertySheetAddPage
 * 
 * This function adds a page widget to the specified location.
 * A position of 0 indicates that the page should be added to the
 * end.
 */
void
XmPropertySheetAddPage(Widget sheet, Widget page, int position)
{
   XmPropertySheetWidget ps = (XmPropertySheetWidget)sheet;
   WidgetList pages;
   WidgetList tabs;
   int page_count;
   XmString tab_string;
   Widget tab;

   pages = ps->property_sheet.pages;
   tabs = ps->property_sheet.tabs;
   page_count = ps->property_sheet.page_count;

   if(XmIsPropertyPage(page) == FALSE)
   {
      XtAppError(XtWidgetToApplicationContext(sheet),
                 "You can only add Widgets that are a"
                 " subclass of PropertyPage.");
      return;
   }
   else if(position < 0 || position > (page_count + 1))
   {
      position = 0;
   }

   /* 
    * Get the dialog title resource from the page widget 
    * and use this as the label text for the push button.
    */

   XtVaGetValues(page, XmNdialogTitle, &tab_string, NULL);
   
   /* FIX ME?
    * create a push button (I remember reading somewhere that labels 
    * can have an activate callback resource...hmm.. I'll look into
    * this later.
    * 
    * Use the title resource to get the text for the push button
    *
    * BTW, the creation of the tab button has to be done at this
    * time because we want the user to be able to add pages on the
    * fly.  Why this would be done I don't know but we will make 
    * an attempt at implementing this.
    */

   tab = XmCreatePushButton(sheet, TAB_RESOURCE_NAME, NULL, 0);
   XtVaSetValues(tab, XmNlabelString, tab_string, NULL);
   XmStringFree(tab_string);

   /*  
    * Add the tab_activate_cb function for handling the activate callback. 
    * Also, the page widget must be sent as the user data for the call.
    */ 
   XtAddCallback(tab, XmNactivateCallback, tab_activate_cb, page);

   if(page_count <= 0)
   {
      pages = (WidgetList) XtMalloc(sizeof(Widget));
      pages[0] = page;
      tabs[0] = tab;
      page_count++;
   }
   /* if position is zero then add page to the end of the list. */
   else if(position == 0)
   {
      pages = (WidgetList) XtRealloc((char *)pages, 
                                     sizeof(Widget)*(page_count+1));
      pages[page_count] = page;
      tabs[page_count] = tab;
      page_count++;
   }
   /* insert page widget into list */
   else
   {
      /* use memmove because memory will overlap */
      pages = (WidgetList) XtRealloc((char *) pages, 
                                     sizeof(Widget)*(page_count+1));
      memmove(/* dest */ &pages[position],
              /* src */ &pages[position-1], 
              (sizeof(Widget)*((page_count+1)-position)));

      tabs = (WidgetList) XtRealloc((char *) tabs, 
                                     sizeof(Widget)*(page_count+1));
      memmove(/* dest */ &tabs[position],
              /* src */ &tabs[position-1], 
              (sizeof(Widget)*((page_count+1)-position)));

      pages[position-1] = page;
      tabs[position-1] = tab;
      page_count++;
   }

   ps->property_sheet.tabs = tabs;
   ps->property_sheet.pages = pages;
   ps->property_sheet.page_count = page_count;
}

/*
 * XmPropertySheetRemovePos
 * 
 * This function removes the page widget from the sheet widget.
 * This function does not delete the page widget.
 */
void
XmPropertySheetRemovePos(Widget sheet, int position)
{
   XmPropertySheetWidget ps = (XmPropertySheetWidget)sheet;
   Widget page, tab;
   WidgetList pages, tabs;
   int page_count;

   pages = ps->property_sheet.pages;
   tabs = ps->property_sheet.tabs;
   page_count = ps->property_sheet.page_count;

   if(position > page_count
      || position <= 0)
   {
      return; /* not in list */
   }

   page = pages[position - 1];
   tab = tabs[position - 1];

   /* adjust list by shifting everything down one in the list */
   memmove(/* dest */ &tabs[position-1],
           /* src */ &tabs[position], 
           (sizeof(Widget)*(page_count-position)));

   page_count--;

   ps->property_sheet.tabs = tabs;
   ps->property_sheet.pages = pages;
   ps->property_sheet.page_count = page_count;

   remove_tab(tab);
   remove_page(page);

   /* was it the active page */
   if (position == ps->property_sheet.active_pos)
   {
      /* 
       * Even though its the active page we are not going to initiate
       * the kill_activate callback  because the user did not select a
       * new tab.  The programmer actually killed it.
       */
      /*
       * At this point it should be up to the programmer to set the new active 
       * page.
       */
      ps->property_sheet.active_pos = 0;
   }
}

/*
 * XmPropertySheetRemovePage
 * 
 * This function actually searches the pages for the specified widget 
 * to determine the pos.  After the position is determined 
 * the XmPropertySheetRemovePos function is called.
 */
void
XmPropertySheetRemovePage(Widget sheet, Widget page)
{
   int pos;
   pos = page_position(sheet, page);
   XmPropertySheetRemovePos(sheet, pos);
}

void 
XmPropertySheetSetActivePage(Widget sheet, Widget page, Boolean notify)
{
   int pos;
   pos = page_position(sheet, page);
   XmPropertySheetSetActivePos(sheet, pos, notify);
}

void 
XmPropertySheetSetActivePos(Widget sheet, int position, Boolean notify)
{
   XmPropertySheetWidget ps = (XmPropertySheetWidget)sheet;
   WidgetList pages, tabs;
   int page_cnt;

   pages = ps->property_sheet.pages;
   tabs = ps->property_sheet.tabs;
   page_cnt = ps->property_sheet.page_count;

   if(position > page_cnt || position <= 0)
   {
      return;
   }
   
   if(notify == TRUE)
   {
      tab_activate_cb(tabs[position-1], pages[position-1], NULL);
   }
   else
   {
      kill_active_page((Widget) sheet);
      set_active_pos((Widget) sheet, position);
   }
}

